{
 ***************************************************************************
 *                                                                         *
 *   This source is free software; you can redistribute it and/or modify   *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This code is distributed in the hope that it will be useful, but      *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
 *   General Public License for more details.                              *
 *                                                                         *
 *   A copy of the GNU General Public License is available on the World    *
 *   Wide Web at <http://www.gnu.org/copyleft/gpl.html>. You can also      *
 *   obtain it by writing to the Free Software Foundation,                 *
 *   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.        *
 *                                                                         *
 ***************************************************************************

  Author: Olivier GUILBAUD

  Abstract:
    This code provide an interface with Common UNIX Printing System (CUPS).
    it is based on cups.h version 1.47
    This code is tested with the 1.1.19 of CUPS
    
  link:
    http://www.cups.org
    http://localhost:631/spm.html (for documentation)
    
  See : /usr/share/doc/cups/licence.txt
  
  History
   sept 14 2003 - Create
   nov  04 2003 - First release
------------------------------------------------------------------------------}
unit cups;

{$mode objfpc}{$H+}
{$PACKRECORDS C}

{$LINKLIB cups}
{$LINKLIB ssl}
{$LINKLIB crypto}
{$LINKLIB nsl}

interface

uses
  Classes, SysUtils,libc,
  cups_ipp,cups_http,cups_ppd;

const
  CUPS_VERSION = 1.0119;
  CUPS_VERSION_MAJOR = 1;
  CUPS_VERSION_MINOR = 1;
  CUPS_VERSION_PATCH = 19;
  CUPS_DATE_ANY = -(1);

  //**** Printer Type/Capability Bits ********
  //not a typedef'd enum so we can OR
  CUPS_PRINTER_LOCAL     = $0000;	// Local printer or class
  CUPS_PRINTER_CLASS     = $0001;	// Printer class
  CUPS_PRINTER_REMOTE    = $0002;	// Remote printer or class
  CUPS_PRINTER_BW        = $0004;	// Can do B&W printing
  CUPS_PRINTER_COLOR     = $0008;	// Can do color printing
  CUPS_PRINTER_DUPLEX    = $0010;	// Can do duplexing
  CUPS_PRINTER_STAPLE    = $0020;	// Can staple output
  CUPS_PRINTER_COPIES    = $0040;	// Can do copies
  CUPS_PRINTER_COLLATE   = $0080;	// Can collage copies
  CUPS_PRINTER_PUNCH     = $0100;	// Can punch output
  CUPS_PRINTER_COVER     = $0200;	// Can cover output
  CUPS_PRINTER_BIND      = $0400;	// Can bind output
  CUPS_PRINTER_SORT      = $0800;	// Can sort output
  CUPS_PRINTER_SMALL     = $1000;	// Can do Letter/Legal/A4
  CUPS_PRINTER_MEDIUM    = $2000;	// Can do Tabloid/B/C/A3/A2
  CUPS_PRINTER_LARGE     = $4000;	// Can do D/E/A1/A0
  CUPS_PRINTER_VARIABLE  = $8000;	// Can do variable sizes
  CUPS_PRINTER_IMPLICIT  = $10000;	// Implicit class
  CUPS_PRINTER_DEFAULT   = $20000;	// Default printer on network
  CUPS_PRINTER_OPTIONS   = $FFFC;	// ~(CLASS | REMOTE | IMPLICIT)

type
  PPPChar = ^PPChar;
  cups_ptype_t = dword; //See bit flags consts
  TFunctionWithParam1 = function (_para1:Pchar):Pchar;

  //Printer Options
  Pcups_option_t= ^cups_option_t;
  PPcups_option_t=^Pcups_option_t;
  cups_option_t = record
                    name  : PChar; //Name of option
                    value : PChar; //Value of option
                  end;

  //Destination
  Pcups_dest_t=^cups_dest_t;
  PPcups_dest_t=^Pcups_dest_t;
  cups_dest_t = record
                  name       : PChar;          //Printer or class name
                  instance   : PChar;          //Local instance name or NULL
                  is_default : Longint;        //Is this printer the default?
                  num_options: Longint;        //Number of options
                  options    : Pcups_option_t; //Options
                end;
  //Job
  Pcups_job_t=^cups_job_t;
  PPcups_job_t=^Pcups_job_t;
  cups_job_t = record
                 id               : longint;       //The job ID
                 dest             : Pchar;         //Printer or class name
                 title            : Pchar;         //Title/job name
                 user             : Pchar;         //User the submitted the job
                 format           : Pchar;         //Document format
                 state            : ipp_jstate_t;  //Job state
                 size             : longint;       //Size in kilobytes
                 priority         : longint;       //Priority (1-100)
                 completed_time   : time_t;        //Time the job was completed
                 creation_time    : time_t;        //Time the job was created
                 processing_time  : time_t;        //Time the job was processed
               end;

  function cupsServer:pchar;cdecl;external;

  function cupsGetDefault:PChar; cdecl; external;
  function cupsGetPPD(printer :Pchar): PChar;cdecl;external;
  function cupsLastError : ipp_status_t; cdecl;external;
  function cupsGetPrinters(printers :PPPchar):longint;cdecl;external;
  function cupsDoFileRequest(http:Phttp_t; request:Pipp_t; const resource:Pchar; const filename:Pchar):Pipp_t;cdecl;external;
  function cupsCancelJob(printer:Pchar; job:longint):longint;cdecl;external;
  function cupsEncryption:http_encryption_t;cdecl;external;
  procedure cupsFreeJobs(num_jobs:longint; jobs:Pcups_job_t);cdecl;external;
  function cupsGetClasses(classes:PPPchar):longint;cdecl;external;
  function cupsGetJobs(jobs:PPcups_job_t; dest:Pchar; myjobs:longint; completed:longint):longint;cdecl;external;
  function cupsPrintFile(printer:Pchar; filename:Pchar; title:Pchar;
      num_options:longint; options:Pcups_option_t):longint;cdecl;external;
  function cupsPrintFiles(printer:Pchar; num_files:longint; files:PPchar;
      title:Pchar; num_options:longint; options:Pcups_option_t):longint;cdecl;external;
  function cupsTempFile(filename:Pchar; len:longint):Pchar;cdecl;external;
  function cupsTempFd(filename:Pchar; len:longint):longint;cdecl;external;
  function cupsAddDest(name:Pchar; instance:Pchar; num_dests:longint;
      dests:PPcups_dest_t):longint;cdecl;external;
  procedure cupsFreeDests(num_dests:longint; dests:Pcups_dest_t);cdecl;external;
  function cupsGetDest(name:Pchar; instance:Pchar;
      num_dests:longint; dests:Pcups_dest_t):Pcups_dest_t;cdecl;external;
  function cupsGetDests(dests:PPcups_dest_t):longint;cdecl;external;
  procedure cupsSetDests(num_dests:longint; dests:Pcups_dest_t);cdecl;external;
  function cupsAddOption(name:Pchar; value:Pchar; num_options:longint; options:PPcups_option_t):longint;cdecl;external;
  procedure cupsEncodeOptions(ipp:Pipp_t; num_options:longint; options:Pcups_option_t);cdecl;external;
  procedure cupsFreeOptions(num_options:longint; options:Pcups_option_t);cdecl;external;
  function cupsGetOption(name:Pchar; num_options:longint; options:Pcups_option_t):Pchar;cdecl;external;
  function cupsParseOptions(arg:Pchar; num_options:longint; options:PPcups_option_t):longint;cdecl;external;
  function cupsMarkOptions(ppd:Pppd_file_t; num_options:longint; options:Pcups_option_t):longint;cdecl;external;
  function cupsGetPassword(prompt:Pchar):Pchar;cdecl;external;
  procedure cupsSetEncryption(e:http_encryption_t);cdecl;external;
  procedure cupsSetPasswordCB(cb: TFunctionWithParam1);cdecl;external;
  procedure cupsSetServer(server:Pchar);cdecl;external;
  procedure cupsSetUser(user:Pchar);cdecl;external;
  function cupsUser:Pchar;cdecl;external;

  function cupsDoRequest(ahttp :Phttp_t; aRequest :Pipp_t; aResource :PChar) : Pipp_t;
implementation

  function cupsDoRequest(ahttp :Phttp_t; aRequest :Pipp_t; aResource :PChar) : Pipp_t;
  begin
    Result:=cupsDoFileRequest(aHttp,aRequest,aResource,nil);
  end;
end.

