{
 ***************************************************************************
 *                                                                         *
 *   This source is free software; you can redistribute it and/or modify   *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This code is distributed in the hope that it will be useful, but      *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
 *   General Public License for more details.                              *
 *                                                                         *
 *   A copy of the GNU General Public License is available on the World    *
 *   Wide Web at <http://www.gnu.org/copyleft/gpl.html>. You can also      *
 *   obtain it by writing to the Free Software Foundation,                 *
 *   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.        *
 *                                                                         *
 ***************************************************************************

  Author: Olivier GUILBAUD

  Abstract:
    This code provide an interface with Common UNIX Printing System (CUPS).
    Module PostScript Printer Description definitions.
    it is based on ppd.h version 1.32.

  link:
    http://www.cups.org

  See : /usr/share/doc/cups/licence.txt

  History
   sept 14 2003 - Create
   nov  04 2003 - First release
------------------------------------------------------------------------------}
unit cups_ppd;

{$PACKRECORDS C}
{$mode objfpc}{$H+}
{$LINKLIB cups}
{$LINKLIB ssl}
{$LINKLIB crypto}
{$LINKLIB nsl}

interface

uses Classes, SysUtils,libc;

const
  //PPD size limits (defined in Adobe spec
  PPD_MAX_NAME = 41;    //Maximum size of name + 1 for nul
  PPD_MAX_TEXT = 81;    //Maximum size of text + 1 for nul
  PPD_MAX_LINE = 256;   //Maximum size of line + 1 for nul
  PPD_VERSION  = '4.3'; //Kept in sync with Adobe version number

Type
  //UI types ...
  ppd_ui_t=(
    PPD_UI_BOOLEAN,		//True or False option
    PPD_UI_PICKONE,		//Pick one from a list
    PPD_UI_PICKMANY		//Pick zero or more from a list
  );

  //Order dependency sections
  ppd_section_t=(
    PPD_ORDER_ANY,		//Option code can be anywhere in the file
    PPD_ORDER_DOCUMENT,		//... must be in the DocumentSetup section
    PPD_ORDER_EXIT,		//... must be sent prior to the document
    PPD_ORDER_JCL,		//... must be sent as a JCL command
    PPD_ORDER_PAGE,		//... must be in the PageSetup section
    PPD_ORDER_PROLOG		//... must be in the Prolog section
  );

  //Colorspaces
  ppd_cs_t=(
    PPD_CS_CMYK := -4,		//CMYK colorspace
    PPD_CS_CMY,			//CMY colorspace
    PPD_CS_GRAY := 1,		//Grayscale colorspace
    PPD_CS_RGB := 3,		//RGB colorspace
    PPD_CS_RGBK,		//RGBK (K = gray) colorspace
    PPD_CS_N			//DeviceN colorspace
   );

  //Status Codes
  ppd_status_t=(
    PPD_OK := 0,		//OK
    PPD_FILE_OPEN_ERROR,	//Unable to open PPD file
    PPD_NULL_FILE,		//NULL PPD file pointer
    PPD_ALLOC_ERROR,		//Memory allocation error
    PPD_MISSING_PPDADOBE4,	//Missing PPD-Adobe-4.x header
    PPD_MISSING_VALUE,		//Missing value string
    PPD_INTERNAL_ERROR,		//Internal error
    PPD_BAD_OPEN_GROUP,		//Bad OpenGroup
    PPD_NESTED_OPEN_GROUP,	//openGroup without a CloseGroup first
    PPD_BAD_OPEN_UI,		//Bad OpenUI/JCLOpenUI
    PPD_NESTED_OPEN_UI,		//OpenUI/JCLOpenUI without a CloseUI/JCLCloseUI first
    PPD_BAD_ORDER_DEPENDENCY,	//Bad OrderDependency
    PPD_BAD_UI_CONSTRAINTS,	//Bad UIConstraints
    PPD_MISSING_ASTERISK,	//Missing asterisk in column 0
    PPD_LINE_TOO_LONG,		//Line longer than 255 chars
    PPD_ILLEGAL_CHARACTER,	//Illegal control character
    PPD_ILLEGAL_MAIN_KEYWORD,	//Illegal main keyword string
    PPD_ILLEGAL_OPTION_KEYWORD,	//Illegal option keyword string
    PPD_ILLEGAL_TRANSLATION	//Illegal translation string
  );

  //PPD Attribute Structure
  Pppd_attr_t  =^ppd_attr_t;
  PPppd_attr_t =^Pppd_attr_t;
  ppd_attr_t = record
    name : array[0..(PPD_MAX_NAME)-1] of char;                //Name of attribute (cupsXYZ)
    spec : array[0..(PPD_MAX_NAME + PPD_MAX_TEXT)-1] of char; //Specifier string, if any
    value: Pchar;                                             //Value string
  end;

  //Option choices
  Pppd_choice_t=^ppd_choice_t;
  PPppd_choice_t=^Pppd_choice_t;
  PPPppd_choice_t=^PPppd_choice_t;
  ppd_choice_t = record
    marked : Char;                              //0 if not selected, 1 otherwise
    choice : Array[0..(PPD_MAX_NAME)-1] of char;//Computer-readable option name
    text   : Array[0..(PPD_MAX_TEXT)-1] of char;//Human-readable option name
    code   : Pchar;                             //Code to send for this option
    option : Pointer;                           //Pointer to parent option structure
  end;

  //Options
  Pppd_option_t=^ppd_option_t;
  ppd_option_t = record
    conflicted : char;                              //0 if no conflicts exist, 1 otherwise
    keyword    : array[0..(PPD_MAX_NAME)-1] of char;//Option keyword name ("PageSize", etc.)
    defchoice  : array[0..(PPD_MAX_NAME)-1] of char;//Default option choice
    text       : array[0..(PPD_MAX_TEXT)-1] of char;//Human-readable text
    ui         : ppd_ui_t;                          //Type of UI option
    section    : ppd_section_t;                     //Section for command
    order      : integer;                           //Order number
    num_choices: LongInt;                           //Number of option choices
    choices    : Pppd_choice_t;                     //Option choices
  end;

  //Groups
   {Group text strings are limited to 39 chars + nul in order to
    preserve binary compatibility and allow applications to get
    the group's keyword name. }
  Pppd_group_str=^ppd_group_str;
  ppd_group_str = record
    text          : array[0..(PPD_MAX_TEXT - PPD_MAX_NAME)-1] of char; //Human-readable group name
    name          : array[0..(PPD_MAX_NAME)-1] of char;                //Group name
    num_options   : longint;                                           //Number of options
    options       : Pppd_option_t;                                     //Options
    num_subgroups : longint;                                           //Number of sub-groups
    subgroups     : Pppd_group_str;                                    //Sub-groups (max depth = 1)
  end;
  ppd_group_t = ppd_group_str;
  Pppd_group_t= ^ppd_group_t;

  //Constraints
  Pppd_const_t=^ppd_const_t;
  ppd_const_t = record
    option1 : array[0..(PPD_MAX_NAME)-1] of char; //First keyword
    choice1 : array[0..(PPD_MAX_NAME)-1] of char; //First option/choice (blank for all)
    option2 : array[0..(PPD_MAX_NAME)-1] of char; //Second keyword
    choice2 : array[0..(PPD_MAX_NAME)-1] of char; //Second option/choice (blank for all)
  end;

  //Page Sizes
  Pppd_size_t=^ppd_size_t;
  ppd_size_t = record
    marked : longint;                            //Page size selected?
    name   : array[0..(PPD_MAX_NAME)-1] of char; //Media size option
    width  : double;                             //Width of media in points
    length : double;                             //Length of media in points
    left   : double;                             //Left printable margin in points
    bottom : double;                             //Bottom printable margin in points
    right  : double;                             //Right printable margin in points
    top    : double;                             //Top printable margin in points
  end;

  //Emulators
  Pppd_emul_t=^ppd_emul_t;
  ppd_emul_t = record
    name : array[0..(PPD_MAX_NAME)-1] of char; //Emulator name
    start: Pchar;                              //Code to switch to this emulation
    stop : Pchar;                              //Code to stop this emulation
  end;

  //sRGB Color Profiles
  Pppd_profile_t=^ppd_profile_t;
  ppd_profile_t = record
    resolution : array[0..(PPD_MAX_NAME)-1] of char;   //Resolution or "-"
    media_type : array[0..(PPD_MAX_NAME)-1] of char;   //Media type of "-"
    density    : double;                               //Ink density to use
    gamma      : double;                               //Gamma correction to use
    matrix     : array[0..2] of array[0..2] of double; //Transform matrix
  end;

  //Files
  Pppd_file_t=^ppd_file_t;
  ppd_file_t = record
    language_level     : longint;                //Language level of device
    color_device       : longint;                //1=color device, 0=grayscale
    variable_sizes     : longint;                //1 = supports variable sizes, 0 = doesn't
    accurate_screens   : longint;                //1 = supports accurate screens, 0 = not
    contone_only       : longint;                //1 = continuous tone only, 0 = not
    landscape          : longint;                //-90 or 90
    model_number       : longint;                //Device-specific model number
    manual_copies      : longint;                //1 = Copies done manually, 0 = hardware
    throughput         : longint;                //Pages per minute
    colorspace         : ppd_cs_t;               //Default colorspace
    patches            : Pchar;                  //Patch commands to be sent to printer
    num_emulations     : longint;                //Number of emulations supported
    emulations         : Pppd_emul_t;            //Emulations and the code to invoke them
    jcl_begin          : Pchar;                  //Start JCL commands
    jcl_ps             : Pchar;                  //Enter PostScript interpreter
    jcl_end            : Pchar;                  //End JCL commands
    lang_encoding      : Pchar;                  //Language encoding
    lang_version       : Pchar;                  //Language version (English, Spanish, etc.)
    modelname          : Pchar;                  //Model name (general)
    ttrasterizer       : Pchar;                  //Truetype rasterizer
    manufacturer       : Pchar;                  //Manufacturer name
    product            : Pchar;                  //Product name (from PS RIP/interpreter)
    nickname           : Pchar;                  //Nickname (specific)
    shortnickname      : Pchar;                  //Short version of nickname
    num_groups         : longint;                //Number of UI groups
    groups             : Pppd_group_t;           //UI groups
    num_sizes          : longint;                //Number of page sizes
    sizes              : Pppd_size_t;            //Page sizes
    custom_min         : array[0..1] of double;  //Minimum variable page size
    custom_max         : array[0..1] of double;  //Maximum variable page size
    custom_margins     : array[0..3] of double;  //Margins around page
    num_consts         : longint;                //Number of UI/Non-UI constraints
    consts             : Pppd_const_t;           //UI/Non-UI constraints
    num_fonts          : longint;                //Number of pre-loaded fonts
    fonts              : PPchar;                 //Pre-loaded fonts
    num_profiles       : longint;                //Number of sRGB color profiles
    profiles           : Pppd_profile_t;         //sRGB color profiles
    num_filters        : longint;                //Number of filters
    filters            : PPchar;                 //Filter strings...
    flip_duplex        : longint;                //1 = Flip page for back sides (New in CUPS 1.1)
    protocols          : Pchar;                  //Protocols (BCP, TBCP) string (New in CUPS 1.1.19)
    pcfilename         : Pchar;                  //PCFileName string
    num_attrs          : longint;                //Number of attributes
    cur_attr           : longint;                //Current attribute
    attrs              : PPppd_attr_t;           //Attributes
  end;

  procedure ppdClose(ppd:Pppd_file_t);cdecl;external;
  function ppdCollect(ppd:Pppd_file_t; section:ppd_section_t; choices:PPPppd_choice_t):longint;cdecl;external;
  function ppdConflicts(ppd:Pppd_file_t):longint;cdecl;external;
  function ppdEmit(ppd:Pppd_file_t; fp:PFILE; section:ppd_section_t):longint;cdecl;external;
  function ppdEmitFd(ppd:Pppd_file_t; fd:longint; section:ppd_section_t):longint;cdecl;external;
  function ppdEmitJCL(ppd:Pppd_file_t; fp:PFILE; job_id:longint; user:Pchar; title:Pchar):longint;cdecl;external;
  function ppdFindChoice(o:Pppd_option_t; option:Pchar):Pppd_choice_t;cdecl;external;
  function ppdFindMarkedChoice(ppd:Pppd_file_t; keyword:Pchar):Pppd_choice_t;cdecl;external;
  function ppdFindOption(ppd:Pppd_file_t; keyword:Pchar):Pppd_option_t;cdecl;external;
  function ppdIsMarked(ppd:Pppd_file_t; keyword:Pchar; option:Pchar):longint;cdecl;external;
  procedure ppdMarkDefaults(ppd:Pppd_file_t);cdecl;external;
  function ppdMarkOption(ppd:Pppd_file_t; keyword:Pchar; option:Pchar):longint;cdecl;external;
  function ppdOpen(fp:PFILE):Pppd_file_t;cdecl;external;
  function ppdOpenFd(fd:longint):Pppd_file_t;cdecl;external;
  function ppdOpenFile(filename:Pchar):Pppd_file_t;cdecl;external;
  function ppdPageLength(ppd:Pppd_file_t; name:Pchar):double;cdecl;external;
  function ppdPageSize(ppd:Pppd_file_t; name:Pchar):Pppd_size_t;cdecl;external;
  function ppdPageWidth(ppd:Pppd_file_t; name:Pchar):double;cdecl;external;

  {New in CUPS 1.1.19}
  function ppdErrorString(status:ppd_status_t):Pchar;cdecl;external;
  function ppdFindAttr(ppd:Pppd_file_t; name:Pchar; spec:Pchar):Pchar;cdecl;external;
  function ppdFindNextAttr(ppd:Pppd_file_t; name:Pchar; spec:Pchar):Pchar;cdecl;external;
  function ppdLastError(line:Plongint):ppd_status_t;cdecl;external;

implementation
end.

